@@ -22,19 +22,19 @@ command_all_parse_args ()
 		exit 1
 		;;
 	esac
+	COMMAND_SHIFT=$(($num_args_init - $#))
 }
 
 # ============= Execute command operations which do not need package tools =============
 command_all_exec_without_pkgtools ()
 {
-	shift || :
 	case $COMMAND_MODE in
 	mount )
-		fs_mount "${SYSTEMBASE}"
+		fs_mount
 		exit
 		;;
 	unmount )
-		fs_unmount "${SYSTEMBASE}"
+		fs_unmount
 		exit
 		;;
 	esac
