@@ -7,11 +7,6 @@
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
-# ============= Set termination messages for special commands =============
-main_set_termination_messages_special ()
-{
-}
-
 # ============= Save the previous configuration if exists =============
 main_save_prev_conf ()
 {
@@ -37,7 +32,7 @@ main_parse_conf ()
 # ============= Check of the mounting/unmounting privilege of the environment =============
 main_check_of_mounting_unmounting_privilege ()
 {
-	if ! misc_chk_mount_privilege || ! misc_chk_unmount_privilege
+	if ! fs_chk_mount_privilege || ! fs_chk_unmount_privilege
 	then
 		temp_terminate_process () { :; }
 		message_echo "ERROR: The current environment does not have the full privilege to mount and unmount file systems." >&2
