@@ -18,6 +18,15 @@ h	help	opt_help_mode	0	1
 h	short-help	opt_help_mode	0	1
 V	show-version	opt_show_version	no	yes
 a	batch-mode	opt_batch_mode	no	yes
+f	forcible-operation	opt_forcible_operation	no	yes
+J	just-save-options	opt_just_save_options	no	yes
 S	no-opening-message	opt_no_opening_message	no	yes
 eof
 }
+
+# ============= Regularize the option value =============
+options_regularize ()
+{
+	opt_packages_dir=`fs_global_path "$opt_packages_dir"`
+	opt_invalidate_mount_privilege=no
+}
