@@ -7,14 +7,18 @@
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
-# ============= Set termination messages for special commands =============
-main_set_termination_messages_special ()
+# ============= Set termination messages which should be shown after mounting file systems =============
+main_inspect_fs_privilege ()
 {
+	misc_chk_privilege || return 0
+	fs_inspect_fs_privilege
+	message_echo
 	temp_terminate_process ()
 	{
 		local errno
 		errno=${1:-0}
-		! fs_chk_unmount && misc_chk_mount_privilege && fs_unmount
+		fs_chk_unmount > /dev/null || ( fs_unmount ) || :
 		temp_terminate_process_common "$errno"
 	}
+	[ $TEMP_IN_TRAP = no ] || temp_terminate_process
 }
