@@ -80,10 +80,13 @@ database_record_failure ()
 		done
 	done
 	rm -f "$nodedir/succeeded_once"
+	fileedit_rm_a_line "$origin" "${DBDIR}/inst_by_port_with_custom_conf.list"
+	fileedit_rm_a_line "$origin" "${DBDIR}/inst_by_port_with_default_conf.list"
+	fileedit_rm_a_line "$origin" "${DBDIR}/installation_complete_by_pkg.list"
 	if [ -z "$clean" -o "@$clean" = @clean ]
 	then
 		message_echo "*** Trying to clean the failed build... (Ignore failures)"
-		env ${MAKE_ENVS} make clean ${MAKE_ARGS} || :
+		database_build_make "$origin" clean | message_cat
 	fi
 	message_echo "*** Skipping this port and proceeding to next forcibly..."
 	message_echo
