@@ -108,25 +108,22 @@ fileedit_manipulate_old_new_lines ()
 	newsrc=$2
 	olddiff=$3
 	newdiff=$4
-	tmpdiff=${TMPDIR}/fileedit_manipulate_old_new_lines::diff
 	[ -e "$oldsrc" ] || oldsrc=/dev/null
 	[ -e "$newsrc" ] || newsrc=/dev/null
-	diff "$oldsrc" "$newsrc" > $tmpdiff && return 1
-	grep '^<' "$tmpdiff" | sed 's/^< //' > $olddiff || :
-	grep '^>' "$tmpdiff" | sed 's/^> //' > $newdiff || :
+	grep -vFx -f "$newsrc" "$oldsrc" > $olddiff 2> /dev/null || :
+	grep -vFx -f "$oldsrc" "$newsrc" > $newdiff 2> /dev/null || :
+	[ `cat "$olddiff" "$newdiff" | wc -l` -gt 0 ]
 }
 
 # ============= Manipulate old lines from an old and a new versions of a file =============
 fileedit_manipulate_old_lines ()
 {
-	local oldsrc newsrc tmpdiff
+	local oldsrc newsrc
 	oldsrc=$1
 	newsrc=$2
-	tmpdiff=${TMPDIR}/fileedit_manipulate_old_lines::diff
 	[ -e "$oldsrc" ] || oldsrc=/dev/null
 	[ -e "$newsrc" ] || newsrc=/dev/null
-	diff "$oldsrc" "$newsrc" > $tmpdiff && return 1
-	grep '^<' "$tmpdiff" | sed 's/^< //'
+	grep -vFx -f "$newsrc" "$oldsrc" 2> /dev/null
 }
 
 # ============= Manipulate new lines from an old and a new versions of a file =============
@@ -135,11 +132,9 @@ fileedit_manipulate_new_lines ()
 	local oldsrc newsrc tmpdiff
 	oldsrc=$1
 	newsrc=$2
-	tmpdiff=${TMPDIR}/fileedit_manipulate_new_lines::diff
 	[ -e "$oldsrc" ] || oldsrc=/dev/null
 	[ -e "$newsrc" ] || newsrc=/dev/null
-	diff "$oldsrc" "$newsrc" > $tmpdiff && return 1
-	grep '^>' "$tmpdiff" | sed 's/^> //'
+	grep -vFx -f "$oldsrc" "$newsrc" 2> /dev/null
 }
 
 # ============= Check whether removed lines exists in updating from an old and a new versions of a file =============
@@ -148,9 +143,7 @@ fileedit_exists_old_lines ()
 	local oldsrc newsrc tmpdiff
 	oldsrc=$1
 	newsrc=$2
-	tmpdiff=${TMPDIR}/fileedit_exists_old_lines::diff
 	[ -e "$oldsrc" ] || oldsrc=/dev/null
 	[ -e "$newsrc" ] || newsrc=/dev/null
-	diff "$oldsrc" "$newsrc" > $tmpdiff && return 1
-	grep -q '^<' "$tmpdiff"
+	grep -qvFx -f "$newsrc" "$oldsrc" 2> /dev/null
 }
