@@ -12,10 +12,35 @@ PKGSYS_CMD_PKG_DELETE='pkg delete'	# Corresponding command for pkg_delete
 PKGSYS_AVR_REFETCH_TIMES_PER_SITE=1	# Average number (integer) of retrials for retrieving package or distfiles per mirror site
 PKGSYS_AVR_REFETCH_TIMES_FOR_CHKSUMERR=2	#  Number (integer) of retrials for check sum error in retrieving a package
 
+# ============= Get the time stamp of the ports tree =============
+pkgsys_get_timestamp_portstree ()
+{
+	stat -t %s -f %m "${PORTS_INDEX_DB}" 2> /dev/null || :
+}
+
+# ============= Update the ports tree =============
+pkgsys_update_portstree ()
+{
+		if [ -e "$PORTSNAP_WORKDIR/INDEX" ]
+		then
+			mode=update
+		else
+			mode=extract
+		fi
+		if [ $opt_batch_mode = no ]
+		then
+			stdout=/dev/stdout
+		else
+			stdout=/dev/null
+		fi
+		portsnap fetch $mode > $stdout
+}
+
 # ============= Check implementation of the ports tree =============
 pkgsys_chk_ports_tree_implementation ()
 {
-	local var tmp_work
+	fs_fix_unionfs_image_if_hidden "${PORTSDIR}/Mk/bsd.port.mk" || :
+	fs_fix_unionfs_image_if_hidden "${PORTSDIR}/Makefile" || :
 	if [ ! -d "${PORTSDIR}" ]
 	then
 		message_echo "ERROR: Ports directory ${PORTSDIR} is not found." >&2
@@ -36,6 +61,7 @@ pkgsys_sysvar ()
 	tmp_work=${TMPDIR}/pkgsys_sysvar:work
 	rm -rf "$tmp_work"
 	mkdir "$tmp_work"
+	fs_fix_unionfs_image_if_hidden "${PORTSDIR}/Mk/bsd.port.mk"
 	make -C "$tmp_work" -f "${PORTSDIR}/Mk/bsd.port.mk" -V "$var" 2> /dev/null
 }
 
@@ -296,6 +322,21 @@ pkgsys_is_legacy_tool_available ()
 	which -s pkg_info
 }
 
+# ============= Load portsnap.conf(5) for important directories  =============
+pkgsys_load_portsnap_conf ()
+{
+	if which -s portsnap
+	then
+		PORTSNAP_WORKDIR=`sed -n '/^[[:space:]]*WORKDIR[[:space:]]*=[[:space:]]*\([^[:space:]#]*\)/s//\1/p' /etc/portsnap.conf || :`
+		PORTSNAP_WORKDIR=${PORTSNAP_WORKDIR:-/var/db/portsnap}
+		PORTSNAP_PORTSDIR=`sed -n '/^[[:space:]]*PORTSDIR[[:space:]]*=[[:space:]]*\([^[:space:]#]*\)/s//\1/p' /etc/portsnap.conf || :`
+		PORTSNAP_PORTSDIR=${PORTSNAP_PORTSDIR:-/usr/ports}
+	else
+		PORTSNAP_WORKDIR=
+		PORTSNAP_PORTSDIR=
+	fi
+}
+
 # ============= Define wrapper commands for hiding the differences between pkg_* tools and pkgng =============
 pkgsys_def_pkgtools ()
 {
@@ -316,15 +357,28 @@ pkgsys_def_pkgtools ()
 		export WITH_PKG=yes
 		export WITH_PKGNG=yes
 		PKGSYS_CMD_PKG_DELETE='pkg delete'
+		pkg_is_tool_ready ()
+		{
+			TMPDIR=/dev/null ASSUME_ALWAYS_YES=yes  PACKAGESITE=file:///nonexistent \
+				pkg info -x 'pkg(-devel)?$' >/dev/null 2>&1
+		}
 		pkg_is_tool_available ()
 		{
+			local dev_out dev_err
 			if [ -x /usr/sbin/pkg ]
 			then
-				pkg -N 2> /dev/null && return
-				env ASSUME_ALWAYS_YES=yes pkg bootstrap -f
-				pkg -N 2> /dev/null
+				dev_out=/dev/stdout
+				dev_err=/dev/stderr
+				if [ $opt_batch_mode = yes ]
+				then
+					dev_out=/dev/null
+					dev_err=/dev/null
+				fi
+				pkg_is_tool_ready && return
+				env ASSUME_ALWAYS_YES=yes pkg bootstrap -f > $dev_out 2> $dev_err
+				pkg_is_tool_ready
 			else
-				which -s pkg && return
+				which -s pkg
 			fi
 		}
 		pkg_info_Ea ()
@@ -357,11 +411,11 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_info_e ()
 		{
-			pkg info -e "$@" 2> /dev/null
+			pkg info -qe "$@" 2> /dev/null
 		}
 		pkg_info_eO ()
 		{
-			pkg info -eO "$@" 2> /dev/null
+			pkg info -qeO "$@" 2> /dev/null
 		}
 		pkg_info_Eg ()
 		{
@@ -379,6 +433,11 @@ pkgsys_def_pkgtools ()
 		{
 			pkg info -ql "$@" 2> /dev/null
 		}
+		pkg_info_qs ()
+		{
+			# Return the total storage space occupied by the installed files in bytes
+			pkg info -qs "$@" 2> /dev/null | sed 's/KiB$/*1024/;s/MiB$/*1024^2/;s/GiB$/*1024^3/;s/B$//' | tr '\n' + | sed 's/+$//' | bc -l
+		}
 		pkg_info_flavor ()
 		{
 			local glob_unflavored
@@ -414,7 +473,7 @@ pkgsys_def_pkgtools ()
 				grep '^[^:]*: missing file ' "$tmp_stderr" | sed 's/^[^:]*: missing file //' || :
 				if grep -q '^pkg: .*: No such file or directory$' "$tmp_stderr"	# For the old specification of pkg(8)
 				then
-					pkg info -ql "$pkg" | while read filepath
+					pkg info -ql "$pkg" 2> /dev/null | while read filepath
 					do
 						[ -e "$filepath" ] || echo "$filepath"
 					done
@@ -430,11 +489,11 @@ pkgsys_def_pkgtools ()
 		pkg_info_gen_pkg_origin_table ()
 		{
 			#	pkg query -g '%n-%v\t%o' \* 2> /dev/null > ${DBDIR}/installed_ports:pkg_vs_origin.tbl
-			pkg info -qa | while read pkgname
+			pkg info -qa 2> /dev/null | while read pkgname
 			do
 				origin=`pkg_info_flavored_origin "$pkgname"`
 				printf '%s\t%s\n' "$pkgname" "$origin"
-			done > ${DBDIR}/installed_ports:pkg_vs_origin.tbl
+			done
 		}
 		pkg_create_b ()
 		{
@@ -442,21 +501,42 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_delete_f ()
 		{
-			pkg delete -fqy "$@"
-			pkg -N 2> /dev/null || return 0	# If pkg(8) is deinstalled successfully
-			pkg info -e "$@" || return 0	# Countermeasure for a bug found for pkg-1.3.4 (at least not until 1.2.7_4)
-			pkg delete -fy "$@"
+			local opt_del opt_quit dev_out dev_err
+			opt_del=
+			[ $opt_no_exec_inst_script = yes ] && opt_del='-D'
+			opt_quit=
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				opt_quit='-q'
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
+			pkg delete -fqy $opt_del "$@" > $dev_out 2> $dev_err || :
+			pkg_is_tool_ready || return 0	# If pkg(8) is deinstalled successfully
+			pkg info -e $opt_quit "$@" > /dev/null 2>&1 || return 0
+			pkg delete -fy $opt_del "$@" > $dev_out 2> $dev_err	# Countermeasure for a bug found for pkg-1.3.4 (at least not until 1.2.7_4)
 		}
 		pkg_add_tools ()
 		{
-			local pkgarc tmp_extract prefix prefix_parent pkg
+			local pkgarc tmp_extract prefix prefix_parent pkg opt_quit dev_out dev_err
 			pkgarc=$1
 			tmp_extract=${TMPDIR}/pkgng:pkg_add_tools:extract
+			opt_quit=
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				opt_quit='-q'
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
 			rm -rf "$tmp_extract"
 			mkdir "$tmp_extract"
-			tar xf "$pkgarc" -C "$tmp_extract"
+			tar xf "$pkgarc" -C "$tmp_extract" > $dev_out 2> $dev_err
 			pkg=`pkgsys_pkgarc_to_pkgname "$pkgarc"`
-			if env ASSUME_ALWAYS_YES=YES $tmp_extract/usr/local/sbin/pkg-static add "$pkgarc"
+			if env ASSUME_ALWAYS_YES=YES $tmp_extract/usr/local/sbin/pkg-static add $opt_quit "$pkgarc" > $dev_out 2> $dev_err
 			then
 				message_echo "INFO: $pkg is successfully registered."
 			else
@@ -464,9 +544,11 @@ pkgsys_def_pkgtools ()
 				return 1
 			fi
 		}
-		pkg_add_f ()
+		pkg_add_f_common ()
 		{
-			local pkgarc pkg pkg_tool pkg_gen
+			local option pkgarc pkg pkg_tool pkg_gen opt_add opt_quit dev_out dev_err
+			option=$1
+			shift || :
 			pkg_tool=
 			pkg_gen=
 			for pkgarc in "$@"
@@ -484,25 +566,42 @@ pkgsys_def_pkgtools ()
 			then
 				rm -rf "${TMPDIR}/pkg_add_f"
 				mkdir -p "${TMPDIR}/pkg_add_f"
-				( cd "${TMPDIR}/pkg_add_f" && ln -s $pkg_gen && env ASSUME_ALWAYS_YES=YES pkg add * )
+				opt_add=
+				[ $opt_no_exec_inst_script = yes ] && opt_add='-I'
+				opt_quit=
+				dev_out=/dev/stdout
+				dev_err=/dev/stderr
+				if [ $opt_batch_mode = yes ]
+				then
+					opt_quit='-q'
+					dev_out=/dev/null
+					dev_err=/dev/null
+				fi
+				( cd "${TMPDIR}/pkg_add_f" && ln -s $pkg_gen && env ASSUME_ALWAYS_YES=YES pkg add -M $option $opt_quit $opt_add * > $dev_out 2> $dev_err )
 			fi
 		}
+		pkg_add_f ()
+		{
+			pkg_add_f_common '' "$@"
+		}
 		pkg_add_fF ()
 		{
-			pkg_add_f "$@"
+			pkg_add_f_common '-f' "$@"
 		}
 		pkg_inst_verify_pkg ()
 		{
 			local pkg pkgarc
 			pkg=$1
 			pkgarc=`pkgsys_pkgname_to_pkgarc "${PKGNG_PKG_CACHEDIR}" "$pkg"` || return
-			tar tf "$pkgarc" > /dev/null 2> /dev/null
+			tar tf "$pkgarc" > /dev/null 2>&1
 		}
 		pkg_inst_remote_fetch ()
 		{
-			local pkg
+			local pkg opt_quit
 			pkg=$1
-			pkg fetch -yU "$pkg"
+			opt_quit=
+			[ $opt_batch_mode = yes ] && opt_quit='-q'
+			pkg fetch -yU $opt_quit "$pkg"
 		}
 		pkg_inst_remote_verify_fetch ()
 		{
@@ -513,13 +612,24 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_inst_remote ()
 		{
-			local pkg pkgarc
+			local pkg pkgarc opt_add opt_quit dev_out dev_err
 			pkg=$1
 			pkg_inst_remote_verify_fetch "$pkg" || return
 			pkgarc=`pkgsys_pkgname_to_pkgarc "${PKGNG_PKG_CACHEDIR}" "$pkg"` || return
 			rm -rf "${TMPDIR}/pkg_inst_remote"
 			mkdir -p "${TMPDIR}/pkg_inst_remote"
-			( cd "${TMPDIR}/pkg_inst_remote" && ln -s "$pkgarc" && env ASSUME_ALWAYS_YES=YES pkg add * )
+			opt_add=
+			[ $opt_no_exec_inst_script = yes ] && opt_add='-I'
+			opt_quit=
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				opt_quit='-q'
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
+			( cd "${TMPDIR}/pkg_inst_remote" && ln -s "$pkgarc" && env ASSUME_ALWAYS_YES=YES pkg add -fM $opt_quit $opt_add * > $dev_out 2> $dev_err )
 		}
 		pkg_inst_wild_verify_pkg ()
 		{
@@ -549,44 +659,48 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_inst_remote_wild ()
 		{
-			local pkg pkgarc
+			local pkg pkgarc opt_add dev_out dev_err
 			pkg=$1
 			if pkg_inst_remote_wild_verify_fetch "$pkg"
 			then
 				pkgarc=`pkgsys_fetch_legacy_remote__fetched_pkg`
 				rm -rf "${TMPDIR}/pkg_inst_remote_wild"
 				mkdir -p "${TMPDIR}/pkg_inst_remote_wild"
-				( cd "${TMPDIR}/pkg_inst_remote_wild" && ln -s "$pkgarc" && pkg_add -ifF * ) || return
+				opt_add=
+				[ $opt_no_exec_inst_script = yes ] && opt_add='-I'
+				dev_out=/dev/stdout
+				dev_err=/dev/stderr
+				if [ $opt_batch_mode = yes ]
+				then
+					dev_out=/dev/null
+					dev_err=/dev/null
+				fi
+				( cd "${TMPDIR}/pkg_inst_remote_wild" && ln -s "$pkgarc" && pkg_add -ifF $opt_add * > $dev_out 2> $dev_err) || return
 				message_echo "INFO: Trying to convert the installed legacy package to pkgng."
-				pkg2ng || :
+				pkg2ng > $dev_out 2> $dev_err || :
 				message_echo "INFO: Checking whether the conversion is successful."
-				pkg info -e "$pkg"
+				pkg info -qe "$pkg" 2> /dev/null
 			fi
 		}
-		pkg_inst_remote_repository_version ()
+		pkg_get_remote_repository_version ()
 		{
-			local origin origin_unflavored pkg pkgarc tmp_pkgcell
+			local origin origin_unflavored pkg
 			origin=$1
 			origin_unflavored=`expr "$origin" : '\([^@]*\)'` || return
 			[ -n "$origin_unflavored" ] || return
-			pkg fetch -yU "$origin_unflavored" || return
+			pkg fetch -qyU "$origin_unflavored" || return
 			pkg=`pkg rquery -U %n-%v "$origin_unflavored"` || return
-			[ -n "$pkg" ] || return
-			pkgarc=`pkgsys_pkgname_to_pkgarc "${PKGNG_PKG_CACHEDIR}" "$pkg"` || return
-			tmp_pkgcell=${TMPDIR}/pkg_inst_remote_any_version:pkgcell
-			rm -rf "$tmp_pkgcell"
-			mkdir -p "$tmp_pkgcell"
-			( cd "$tmp_pkgcell" && ln -s "$pkgarc" && env ASSUME_ALWAYS_YES=YES pkg add * )
+			echo "$pkg"
 		}
 		pkg_get_pkgs_timestamps ()
 		{
-			pkg query '%n-%v\tng-epoch:%t' "$@"
+			pkg query '%n-%v\tng-epoch:%t' "$@" 2> /dev/null
 		}
 		pkg_get_pkg_timestamp ()
 		{
 			local pkg
 			pkg=$1
-			pkg query '%t' "$pkg"
+			pkg query '%t' "$pkg" 2> /dev/null
 		}
 		pkg_loadconf ()
 		{
@@ -621,8 +735,9 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_rescue_tools ()
 		{
-			local origin_portsmgmt packagepath checksumpath pkgname is_successful
+			local origin_portsmgmt packagepath checksumpath pkgname is_successful dev_out dev_err
 			origin_portsmgmt=`pkgsys_portsmgmt_pkg`
+			pkg_is_tool_ready && return
 			packagepath=`pkgsys_get_backup_pkg "$origin_portsmgmt"` && \
 				pkg_add_tools "$packagepath" && return
 			pkg_is_tool_available && return
@@ -637,6 +752,13 @@ pkgsys_def_pkgtools ()
 			grep -Ev '^[[:space:]]*WITH_PKG(|NG)=' /etc/make.conf > ${TMPDIR}/make.conf 2> /dev/null || :
 			echo WITHOUT_PKG=yes >> ${TMPDIR}/make.conf
 			echo WITHOUT_PKGNG=yes >> ${TMPDIR}/make.conf
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
 			( set -e
 				unset WITH_PKG
 				unset WITH_PKGNG
@@ -644,14 +766,27 @@ pkgsys_def_pkgtools ()
 				unset WITHOUT_PKGNG
 				message_echo "INFO: Attempting deinstallation of pkg(8) to make sure."
 				portsmgmt_port_path=`pkgsys_get_portpath_from_origin "$origin_portsmgmt"`
-				env __MAKE_CONF="${TMPDIR}/make.conf" make -C "$portsmgmt_port_path" deinstall || :
+				fs_fix_unionfs_image_if_hidden "${TMPDIR}/make.conf"
+				env __MAKE_CONF="${TMPDIR}/make.conf" make -C "$portsmgmt_port_path" deinstall > $dev_out 2> $dev_err || :
 				message_echo "INFO: Attempting (re)installation by pkg(8)."
-				env __MAKE_CONF="${TMPDIR}/make.conf" make -C "$portsmgmt_port_path" reinstall clean
+				env __MAKE_CONF="${TMPDIR}/make.conf" make -C "$portsmgmt_port_path" reinstall clean > $dev_out 2> $dev_err
 			) && {
-				pkg2ng || :
+				pkg2ng > $dev_out 2> $dev_err || :
 				pkg_is_tool_available
 			}
 		}
+		pkg_update_pkgrepository ()
+		{
+			local opts
+			pkg_rescue_tools || return 0
+			if [ $opt_batch_mode = no ]
+			then
+				opts=
+			else
+				opts='-q'
+			fi
+			pkg update $opts
+		}
 		if ! pkg_rescue_tools
 		then
 			message_echo "WARNING: Pkgng is still missing, but continuing for the time being." >&2
@@ -666,6 +801,10 @@ pkgsys_def_pkgtools ()
 		unset WITH_PKGNG
 		PKGSYS_USE_PKGNG=no
 		PKGSYS_CMD_PKG_DELETE='pkg_delete'
+		pkg_is_tool_ready ()
+		{
+			pkgsys_is_legacy_tool_available
+		}
 		pkg_is_tool_available ()
 		{
 			pkgsys_is_legacy_tool_available
@@ -700,7 +839,7 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_info_e ()
 		{
-			pkg_info -e "$@" 2> /dev/null
+			pkg_info -qe "$@" 2> /dev/null
 		}
 		pkg_info_eO ()
 		{
@@ -722,9 +861,14 @@ pkgsys_def_pkgtools ()
 		{
 			pkg_info -qL "$@" 2> /dev/null
 		}
+		pkg_info_qs ()
+		{
+			# Return the total storage space occupied by the installed files in bytes
+			pkg_info -qs "$@" 2> /dev/null | sed 's/[^0-9]*/*1024/' | tr '\n' + | sed 's/+$//' | bc -l
+		}
 		pkg_info_flavor ()
 		{
-			pkg_info -e "$@" 2> /dev/null && echo
+			pkg_info -qe "$@" 2> /dev/null && echo
 			:
 		}
 		pkg_info_flavored_origin ()
@@ -759,29 +903,67 @@ pkgsys_def_pkgtools ()
 			do
 				origin=`pkg_info -qo "$pkg" 2> /dev/null`
 				printf '%s\t%s\n' "$pkg" "$origin"
-			done > ${DBDIR}/installed_ports:pkg_vs_origin.tbl
+			done
 		}
 		pkg_create_b ()
 		{
-			pkg_create -b "$@"
+			local dev_out dev_err
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
+			pkg_create -b "$@" > $dev_out 2> $dev_err
 		}
 		pkg_delete_f ()
 		{
-			pkg_delete -f "$@"
+			local opt_del dev_out dev_err
+			opt_del=
+			[ $opt_no_exec_inst_script = yes ] && opt_del='-D'
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
+			pkg_delete -f $opt_del "$@" > $dev_out 2> $dev_err
 		}
 		pkg_add_f ()
 		{
+			local opt_add dev_out dev_err
 			rm -rf "${TMPDIR}/pkg_add_f"
 			mkdir -p "${TMPDIR}/pkg_add_f"
 			ln -s "$@" "${TMPDIR}/pkg_add_f"
-			( cd "${TMPDIR}/pkg_add_f" && pkg_add -if * )
+			opt_add=
+			[ $opt_no_exec_inst_script = yes ] && opt_add='-I'
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
+			( cd "${TMPDIR}/pkg_add_f" && pkg_add -if $opt_add * > $dev_out 2> $dev_err )
 		}
 		pkg_add_fF ()
 		{
+			local opt_add dev_out dev_err
 			rm -rf "${TMPDIR}/pkg_add_f"
 			mkdir -p "${TMPDIR}/pkg_add_f"
 			ln -s "$@" "${TMPDIR}/pkg_add_f"
-			( cd "${TMPDIR}/pkg_add_f" && pkg_add -ifF * )
+			opt_add=
+			[ $opt_no_exec_inst_script = yes ] && opt_add='-I'
+			dev_out=/dev/stdout
+			dev_err=/dev/stderr
+			if [ $opt_batch_mode = yes ]
+			then
+				dev_out=/dev/null
+				dev_err=/dev/null
+			fi
+			( cd "${TMPDIR}/pkg_add_f" && pkg_add -ifF $opt_add * > $dev_out 2> $dev_err )
 		}
 		pkg_inst_verify_pkg ()
 		{
@@ -805,7 +987,7 @@ pkgsys_def_pkgtools ()
 		}
 		pkg_inst_remote ()
 		{
-			local pkg pkgarc
+			local pkg pkgarc opt_add dev_out dev_err
 			pkg=$1
 			if pkg_inst_remote_verify_fetch "$pkg"
 			then
@@ -813,7 +995,16 @@ pkgsys_def_pkgtools ()
 				rm -rf "${TMPDIR}/pkg_inst_remote"
 				mkdir -p "${TMPDIR}/pkg_inst_remote"
 				ln -s "$@" "${TMPDIR}/pkg_add_f"
-				( cd "${TMPDIR}/pkg_inst_remote" && ls -s "$pkgarc" && pkg_add -ifF * )
+				opt_add=
+				[ $opt_no_exec_inst_script = yes ] && opt_add='-I'
+				dev_out=/dev/stdout
+				dev_err=/dev/stderr
+				if [ $opt_batch_mode = yes ]
+				then
+					dev_out=/dev/null
+					dev_err=/dev/null
+				fi
+				( cd "${TMPDIR}/pkg_inst_remote" && ls -s "$pkgarc" && pkg_add -ifF $opt_add * > $dev_out 2> $dev_err )
 			fi
 		}
 		pkg_inst_wild_verify_pkg ()
@@ -832,35 +1023,31 @@ pkgsys_def_pkgtools ()
 		{
 			pkg_inst_remote "$@"
 		}
-		pkg_inst_remote_repository_version ()
+		pkg_get_remote_repository_version ()
 		{
-			local origin checksumpath version_regexp pkgbase_regexp
+			local origin checksumpath version_regexp pkgbase_regexp pkg
 			origin=$1
-			pkgsys_ready_checksum_file || return
+			pkgsys_ready_checksum_file > /dev/null || return
 			checksumpath=`pkgsys_ready_checksum_file__fetched_file`
 			version_regexp=`database_build_make "$origin" -V PORTVERSION | str_escape_regexp_filter`
 			pkgbase_regexp=`database_build_get_new_pkgname "$origin" | sed -E "s/$version_regexp$//" | str_escape_regexp_filter`
-			sed -nE "s/^MD5[[:space:]]*\(($pkgbase_regexp.*)\.tbz\)[[:space:]]*=.*/\1/p" "$checksumpath" | while read pkg
-			do
-				pkg_inst_remote "$pkg" && return
-			done
+			pkg=`sed -nE "s/^MD5[[:space:]]*\(($pkgbase_regexp.*)\.tbz\)[[:space:]]*=.*/\1/p" "$checksumpath" | head -n 1`
+			echo "$pkg"
 		}
 		pkg_get_pkgs_timestamps ()
 		{
-			ls -lD legacy:%s ${PKG_DBDIR} | \
-				sed -E 's/[[:space:]]+/ /g' | cut -w -f 6,7 | grep -v '^$' | \
-				sed -E 's/^([^[:space:]]*)[[:space:]]*([^[:space:]]*)/\2 \1/' | tr ' ' '\t'
+			stat -t %s -f $'legacy:%m\t%N' ${PKG_DBDIR} 2> /dev/null || :
 		}
 		pkg_get_pkg_timestamp ()
 		{
 			local pkg portdb
 			pkg=$1
 			portdb=`echo "$pkg" | sed 's/-[0-9].*//'`
-			ls -lD legacy:%s "${PKG_DBDIR}/$portdb" | \
-				sed -E 's/[[:space:]]+/ /g' | cut -w -f 6 | grep -v '^$' 
+			stat -t %s -f %m "${PKG_DBDIR}/$portdb" 2> /dev/null || :
 		}
 		pkg_loadconf () { :; }
 		pkg_rescue_tools () { :; }
+		pkg_update_pkgrepository () { :; }
 	fi
 }
 
@@ -930,6 +1117,21 @@ pkgsys_get_installed_pkg_from_origin ()
 	:
 }
 
+# ============= Get the installed package name from glob patterns =============
+pkgsys_get_installed_pkg_from_glob ()
+{
+	local glob regexp
+	for glob in "$@"
+	do
+		if regexp=`expr "$glob" : ':\(.*\)'`
+		then
+			pkg_info_Ex "$regexp"
+		else
+			pkg_info_Eg "$glob"
+		fi
+	done | sort -u
+}
+
 # ============= Check existence of an installed package for a flavored origin =============
 pkgsys_exists_from_orig ()
 {
@@ -942,6 +1144,14 @@ pkgsys_exists_from_orig ()
 	[ "x$flavor_origin" = "x$flavor_pkg" ]
 }
 
+# ============= Generate the package names vs origins table at the initial state =============
+pkgsys_gen_init_pkg_origin_table ()
+{
+	[ -e "${DBDIR}/installed_ports:pkg_vs_origin.tbl" ] && return
+	pkg_info_gen_pkg_origin_table > ${DBDIR}/installed_ports:pkg_vs_origin.tbl.tmp || :
+	mv "${DBDIR}/installed_ports:pkg_vs_origin.tbl.tmp" "${DBDIR}/installed_ports:pkg_vs_origin.tbl"
+}
+
 # ============= Check existence of initially or currently installed package for a flavored origin =============
 pkgsys_exists_or_existed_from_orig ()
 {
@@ -983,7 +1193,7 @@ pkgsys_get_init_pkg_from_orig ()
 # ============= Get the package name of this utility =============
 pkgsys_get_my_current_pkg ()
 {
-	pkg_info_Ex "${APPNAME}-[0-9].*"
+	pkg_info_Ex "${PROGRAM}-[0-9].*"
 }
 
 # ============= Get the flavored origin of this utility =============
@@ -1090,6 +1300,7 @@ pkgsys_eval_ports_glob ()
 					
 				grep -E "$glob_regexp_unflavored" "$unflavored_origlist" 2>&1 | while read origin_unflavored
 				do
+					fs_fix_unionfs_image_if_hidden "${PORTSDIR}/$origin_unflavored"
 					make -C "${PORTSDIR}/$origin_unflavored" -V FLAVORS 2> /dev/null | \
 						tr '[:space:]' '\n' | grep -v '^$' | grep -E "$glob_regexp_flavor" | sed -E "s|^|$origin_unflavored@|"
 				done
@@ -1101,6 +1312,7 @@ pkgsys_eval_ports_glob ()
 				grep -E "$glob_regexp" "$unflavored_origlist" 2>&1 | while read origin_unflavored
 				do
 					origin_unflavored_rpl=`str_escape_replaceval "$origin_unflavored"`
+					fs_fix_unionfs_image_if_hidden "${PORTSDIR}/$origin_unflavored"
 					make -C "${PORTSDIR}/$origin_unflavored" -V FLAVORS 2> /dev/null | \
 						tr '[:space:]' '\n' | grep -v '^$' > $tmp_flavors || :
 					if [ `wc -l < $tmp_flavors` -gt 0 ]
@@ -1117,7 +1329,7 @@ pkgsys_eval_ports_glob ()
 				cut -f 2 "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null
 			} | grep -E "$glob_regexp_allflavors" 2>&1 || :
 		else
-			if expr "$glob" : '^[a-z][a-zA-Z0-9_.+-]*[a-zA-Z0-9_.+]$' > /dev/null 2>&1 && \
+			if expr "$glob" : '[a-z][a-zA-Z0-9_.+-]*[a-zA-Z0-9_.+]$' > /dev/null 2>&1 && \
 				[ `expr "$glob" : '.*-[0-9]' 2>&1` -eq 0 ]
 			then
 				glob_regexp2=`expr "$glob_regexp" : '\(.*\)\$$' 2>&1 || :`'-[0-9]'
@@ -1131,6 +1343,7 @@ pkgsys_eval_ports_glob ()
 			done | while read origin_unflavored
 			do
 				origin_unflavored_rpl=`str_escape_replaceval "$origin_unflavored"`
+				fs_fix_unionfs_image_if_hidden "${PORTSDIR}/$origin_unflavored"
 				make -C "${PORTSDIR}/$origin_unflavored" -V FLAVORS 2> /dev/null | \
 					tr '[:space:]' '\n' | grep -v '^$' > $tmp_flavors || :
 				[ `wc -l < $tmp_flavors` -gt 0 ] || echo > $tmp_flavors
@@ -1152,8 +1365,8 @@ pkgsys_eval_ports_glob ()
 					fi
 				fi
 			done || :
-			glob_regexp2=`expr "$glob_regexp" : '\(.*\)\$$' 2>&1 || :`'[[:space:]]'
-			sed -n -E "/$glob_regexp2/p" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
+			glob_regexp2=`echo "$glob_regexp" | sed -E 's/\$*$//' 2>&1 || :`'[[:space:]]'
+			grep -E "$glob_regexp2" "${DBDIR}/installed_ports:pkg_vs_origin.tbl" 2> /dev/null \
 				| cut -f 2 || :
 			pkg_info_Ex "$glob_regexp" | while read pkgname
 			do
@@ -1352,19 +1565,25 @@ pkgsys_eval_installed_pkgs_globs ()
 # ============= Get glob patterns of conflicting packages of a port =============
 pkgsys_get_conflicting_pkgs_patterns ()
 {
-	local mode origin conflicts
+	local mode origin conflicts conflicts_makevar conflicts_config
 	mode=$1
 	origin=$2
 	conflicts=`database_query_get_makevar_val "$origin" CONFLICTS`
 	case $mode in
 	build )
-		conflicts=$conflicts' '`database_query_get_makevar_val "$origin" CONFLICTS_BUILD`
+		conflicts_makevar=`database_query_get_makevar_val "$origin" CONFLICTS_BUILD`
+		conflicts_config=`database_query_get_config_val "$origin" BUILDCONFLICT`
 		;;
 	install )
-		conflicts=$conflicts' '`database_query_get_makevar_val "$origin" CONFLICTS_INSTALL`
+		conflicts_makevar=`database_query_get_makevar_val "$origin" CONFLICTS_INSTALL`
+		conflicts_config=`database_query_get_config_val "$origin" INSTCONFLICT`
+		;;
+	*)
+		conflicts_makevar=
+		conflicts_config=
 		;;
 	esac
-	echo "$conflicts" | tr ' ' '\n' | grep -v '^$' || :
+	echo "$conflicts $conflicts_makevar $conflicts_config" | tr ' ' '\n' | grep -v '^$' | sort -u
 }	
 
 # ============= Get conflicting installed packages of a port =============
@@ -1376,7 +1595,7 @@ pkgsys_get_conflicting_installed_pkgs ()
 	tmp_conflicts=${TMPDIR}/pkgsys_get_conflicting_installed_pkgs::conflicts
 	pkgsys_get_conflicting_pkgs_patterns "$mode" "$origin" | while read pkg_pattern
 	do
-		pkg_info_Eg "$pkg_pattern" || :
+		pkgsys_get_installed_pkg_from_glob "$pkg_pattern" || :
 	done > $tmp_conflicts
 	cat "$tmp_conflicts"
 	[ `wc -l < $tmp_conflicts` -gt 0 ]
@@ -1482,7 +1701,8 @@ pkgsys_get_current_port_oprions_timestamp ()
 # ============= Get the saved all timestamp information of port options =============
 pkgsys_get_saved_port_oprions_timestamps_all ()
 {
-	cat "${DBDIR}/ls_dbdir/"*.log > $tmp_ls.db 2> /dev/null || :
+	mkdir -p "${DBDIR}/ls_dbdir"
+	cat "${DBDIR}/ls_dbdir/"*.log 2> /dev/null || :
 }
 
 # ============= Convert a list of port origins to port options timestamp log names =============
@@ -1491,18 +1711,26 @@ pkgsys_conv_portorigin_to_port_oprion_timestamp_logname ()
 	sed 's|/|_|'
 }
 
+# ============= Get the file name of the port options database of a port =============
+pkgsys_get_port_oprion_database ()
+{
+	local origin
+	origin=$1
+	if pkgsys_is_dialog4ports_used
+	then
+		echo "$origin" | sed 's|/|_|;s/@.*//'
+	else
+		database_build_make "$origin" -V UNIQUENAME
+	fi
+}
+
 # ============= Save the timestamp information of port options of a port =============
 pkgsys_save_port_oprion_timestamp ()
 {
 	local origin portoptlog portoptdb
 	origin=$1
 	portoptlog=`echo "$origin" | pkgsys_conv_portorigin_to_port_oprion_timestamp_logname`
-	if pkgsys_is_dialog4ports_used
-	then
-		portoptdb=`echo "$portoptlog" | sed 's/@.*//'`
-	else
-		portoptdb=`database_build_make "$origin" -V UNIQUENAME`
-	fi
+	portoptdb=`pkgsys_get_port_oprion_database "$origin"`
 	mkdir -p "${DBDIR}/ls_dbdir"
 	pkgsys_get_current_port_oprions_timestamp "$portoptdb" > ${DBDIR}/ls_dbdir/$portoptlog.log 2> /dev/null || :
 }
@@ -1526,24 +1754,35 @@ pkgsys_get_changed_port_oprions ()
 	fi
 }
 
-# ============= Convert a list of port origins to port options database names =============
-pkgsys_conv_portorigins_to_portoptiondbs ()
+# ============= Convert a list of port options database names to port globs =============
+pkgsys_conv_portoptiondbs_to_globs ()
 {
 	if pkgsys_is_dialog4ports_used
 	then
 		sed 's|_|/|'
 	else
-		cat "$tmp_ls.diff"
+		cat
 	fi
 }
-	
+
 # ============= Register nonexistent port options databases =============
 pkgsys_register_list_nonexistent_portopriondb ()
 {
 	local dbpath
 	dbpath=${DBDIR}/ls_dbdir/%NONEXISTENT%.log
+	mkdir -p "${DBDIR}/ls_dbdir"
 	cat > $dbpath.new
 	touch  "$dbpath"
 	sort -u "$dbpath.new" "$dbpath" > $dbpath.tmp
 	mv "$dbpath.tmp" "$dbpath"
 }
+
+# ============= Order a list of installed packages by storage space occupied by the installed files =============
+pkgsys_order_pkgs_by_size ()
+{
+	while read pkg
+	do
+		size=`pkg_info_qs "$pkg" || :`
+		[ -n "$size" ] && echo "$size"$'\t'"$pkg"
+	done | sort -g | cut -f 2
+}
