@@ -12,6 +12,7 @@ PROGRAM_NUM_STEPS=	# [Used in _program_exec_restartable_loop_operation__routine]
 PROGRAM_STEP_PROGRESS=	# [Used in _program_exec_restartable_loop_operation__routine] Current progress as the finished number of routine steps
 PROGRAM_STEP_COUNTER=	# [Used in _program_exec_restartable_loop_operation__routine] Counter string indicating the current progress
 PROGRAM_STAGETAG=	# [Used in _program_exec_restartable_loop_operation__routine] Current stage tag
+PROGRAM_STEP_COUNTER_LAST_SKIPPED=no	# Whether the last iteration was skipped
 
 # ============= Check completion of a stage =============
 program_chk_stage_complete ()
@@ -30,6 +31,15 @@ program_chk_stage_loop_complete ()
 	[ -e "${DBDIR}/stage.reinit_loop/$stagetag" ]
 }
 
+# ============= Check completion of a loop in a stage =============
+program_chk_stage_loop_complete ()
+{
+	local stagetag
+	stagetag=$1
+	program_chk_stage_complete "$stagetag" && return
+	[ -e "${DBDIR}/stage.reinit_loop/$stagetag" ]
+}
+
 # ============= Register completion of a stage =============
 program_register_stage_complete ()
 {
@@ -130,7 +140,7 @@ program_reset_exec_restartable_loop_operation
 # ============= Execute a restartable loop operation =============
 program_exec_restartable_loop_operation ()
 {
-	local inputdb tmp_diff looplist nlines PROGRAM_NUM_STEPS PROGRAM_STEP_PROGRESS iline item
+	local inputdb tmp_diff looplist nlines iline item
 	inputdb=$1
 	tmp_diff=${TMPDIR}/program_exec_restartable_loop_operation:diff
 	looplist=${DBDIR}/stage.loop_list/$inputdb
@@ -138,10 +148,8 @@ program_exec_restartable_loop_operation ()
 	then
 		if [ -f "$looplist.remain" -a ! -e "${DBDIR}/stage.reinit_loop/$PROGRAM_STAGETAG" ]
 		then
-			cp "$looplist.remain" "$looplist.remain.tmp"
-			cat "$looplist.remain.new" >> $looplist.remain.tmp
-			sort -u "$looplist.remain.tmp" >  $looplist.remain.tmp2
-			mv "$looplist.remain.tmp2" "$looplist.remain"
+			sort -u "$looplist.remain" "$looplist.remain.new" > $looplist.remain.tmp
+			mv "$looplist.remain.tmp" "$looplist.remain"
 		fi
 		cat "$looplist" 2> /dev/null > $looplist.tmp || :
 		mv "$looplist.tmp" "$looplist.prev"
@@ -159,13 +167,18 @@ program_exec_restartable_loop_operation ()
 	nlines=`wc -l < ${TMPDIR}/$inputdb.input`
 	PROGRAM_STEP_PROGRESS=$(($PROGRAM_NUM_STEPS-$nlines))
 	iline=1
+	PROGRAM_STEP_COUNTER_LAST_SKIPPED=no
 	while [ $iline -le $nlines ]
 	do
 		item=`sed -n ${iline}p "${TMPDIR}/$inputdb.input"`
 		iline=$(($iline+1))
 		PROGRAM_STEP_PROGRESS=$(($PROGRAM_STEP_PROGRESS+1))
 		PROGRAM_STEP_COUNTER="[$PROGRAM_STEP_PROGRESS/$PROGRAM_NUM_STEPS $(($PROGRAM_STEP_PROGRESS*100/$PROGRAM_NUM_STEPS))%]"
-		grep -q -Fx "$item" "$looplist.remain" 2> /dev/null || continue
+		if ! grep -q -Fx "$item" "$looplist.remain" 2> /dev/null
+		then
+			[ $iline -gt $nlines ] && PROGRAM_STEP_COUNTER_LAST_SKIPPED=yes
+			continue
+		fi
 		_program_exec_restartable_loop_operation__routine "$item"
 		fileedit_rm_a_line "$item" "$looplist.remain"
 	done
