@@ -72,3 +72,15 @@ str_concat_items_for_sentence ()
 {
 	sed 's/$/,/' | tr '\n' ' ' | sed -E 's/, $//;s/, ([^ ]*)$/ and \1/'
 }
+
+# ============= Regularize a file path for df and /etc/fstab (by stream interface) =============
+str_regularize_df_path_filter ()
+{
+	sed 's|//*|/|g;s|/$||;s/[[:space:]]/\\040/g'
+}
+
+# ============= Regularize a file path for df and /etc/fstab (by argument interface) =============
+str_regularize_df_path ()
+{
+	echo -n "$*" | str_regularize_df_path_filter
+}
