@@ -18,5 +18,13 @@ usage_short_content () { :; }
 # ============= Output a long help =============
 usage_long ()
 {
-	zcat "${MYPREFIX}/man/man8/${APPNAME}.8.gz" | nroff -man
+	local manpage_path
+	manpage_path=${MYPREFIX}/man/man8/${APPNAME}.8
+	if [ -e "$manpage_path.gz" ]
+	then
+		zcat "$manpage_path.gz"
+	elif [ -e "$manpage_path" ]
+	then
+		cat "$manpage_path"
+	fi | nroff -man
 }
