@@ -7,11 +7,6 @@
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
-# ============= Set termination messages for special commands =============
-main_set_termination_messages_special ()
-{
-}
-
 # ============= Save the previous configuration if exists =============
 main_save_prev_conf ()
 {
